#pragma once

#include "fastfetch.h"

#define FF_TITLE_MODULE_NAME "Title"

void ffPrintTitle(FFTitleOptions* options);
void ffInitTitleOptions(FFTitleOptions* options);
void ffDestroyTitleOptions(FFTitleOptions* options);
