#pragma once

#include "fastfetch.h"

#define FF_EDITOR_MODULE_NAME "Editor"

void ffPrintEditor(FFEditorOptions* options);
void ffInitEditorOptions(FFEditorOptions* options);
void ffDestroyEditorOptions(FFEditorOptions* options);
