#pragma once

#include "fastfetch.h"

#define FF_WMTHEME_MODULE_NAME "WMTheme"

void ffPrintWMTheme(FFWMThemeOptions* options);
void ffInitWMThemeOptions(FFWMThemeOptions* options);
void ffDestroyWMThemeOptions(FFWMThemeOptions* options);
